@@ -14,7 +14,9 @@ import os
 import threading
 import time
 import sys
-from xml.dom import minidom
+
+# Need to disable pytype for minidom due to https://github.com/google/pytype/issues/1130
+from xml.dom import minidom  # pytype: disable=pyi-error
 from xml.etree import ElementTree
 import zipfile
 
@@ -180,7 +182,6 @@ class OutputHandler(object):
             runSet.xml.set("error", msg or "unknown error")
 
     def store_header_in_xml(self, version, memlimit, timelimit, corelimit):
-
         # store benchmarkInfo in XML
         self.xml_header = ElementTree.Element(
             "result",
@@ -504,7 +505,6 @@ class OutputHandler(object):
         # format numbers, number_of_digits is optional, so it can be None
         for column in run.columns:
             if column.number_of_digits is not None:
-
                 # if the number ends with "s" or another letter, remove it
                 if (not column.value.isdigit()) and column.value[-2:-1].isdigit():
                     column.value = column.value[:-1]
@@ -699,7 +699,7 @@ class OutputHandler(object):
             return
 
         if hasattr(value, "__getitem__") and not isinstance(value, (str, bytes)):
-            value = ",".join(map(str, value))
+            value = ",".join(map(str, value))  # pytype: disable=wrong-arg-types
         elif isinstance(value, datetime.datetime):
             value = value.isoformat()
 
