@@ -181,8 +181,8 @@ def _get_cpu_cores_per_run0(
         for core in unused_cores:
             siblings_of_core.pop(core)
         logging.debug(
-            f"Running in no-hyperthreading mode, "
-            f"avoiding the use of CPU cores {unused_cores}"
+            "Running in no-hyperthreading mode, avoiding the use of CPU cores %s",
+            unused_cores,
         )
 
     unit_size = len(next(iter(cores_of_unit.values())))  # Number of units per core
@@ -353,7 +353,8 @@ def get_memory_banks_per_run(coreAssignment, cgroups):
 def _get_memory_banks_listed_in_dir(path):
     """Get all memory banks the kernel lists in a given directory.
     Such a directory can be /sys/devices/system/node/ (contains all memory banks)
-    or /sys/devices/system/cpu/cpu*/ (contains all memory banks on the same NUMA node as that core)."""
+    or /sys/devices/system/cpu/cpu*/ (contains all memory banks on the same NUMA node as that core).
+    """
     # Such directories contain entries named "node<id>" for each memory bank
     return [int(entry[4:]) for entry in os.listdir(path) if entry.startswith("node")]
 
