@@ -217,7 +217,7 @@ def _get_task_counts(rows):
     max_score = None
     for row in rows:
         if not row.id.property:
-            logging.info("Missing property for task %s.", row.id)
+            logging.debug("Missing property for task %s.", row.id)
             continue
         expected_result = row.id.expected_result
         if not expected_result:
@@ -226,7 +226,9 @@ def _get_task_counts(rows):
             count_true += 1
         elif expected_result.result is False:
             count_false += 1
-        row_max_score = row.id.property.max_score(expected_result)
+        row_max_score = row.id.property.max_score(
+            expected_result, row.id.witness_category
+        )
         if row_max_score is not None:
             max_score = row_max_score + (max_score or 0)
 
