@@ -24,7 +24,7 @@ class Tool(benchexec.tools.template.BaseTool):
     def determine_result(self, returncode, returnsignal, output, isTimeout):
         output = "\n".join(output)
         if ((returnsignal == 9) or (returnsignal == 15)) and isTimeout:
-            status = "TIMEOUT"
+            status = result.RESULT_TIMEOUT
         elif returnsignal == 9:
             status = "KILLED BY SIGNAL 9"
         elif "RESULT: TRUE" in output:
